#!c:/Python24/python.exe
#
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is standalone Firefox Windows performance test.
#
# The Initial Developer of the Original Code is Google Inc.
# Portions created by the Initial Developer are Copyright (C) 2006
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Annie Sullivan <annie.sullivan@gmail.com> (original author)
#   Elizabeth Chak <elichak@gmail.com> (Seneca@York)
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

"""Runs extension performance tests.

   This file runs Ts (startup time) and Tp (page load time) tests
   for an extension with different profiles.  It was originally
   written for the Google Toolbar for Firefox; to make it work for
   another extension modify get_xpi.py.  To change the preferences
   that are set on the profiles that are tested, edit the arrays in
   the main function below.
"""

__author__ = 'annie.sullivan@gmail.com (Annie Sullivan)'
__contributor__ = 'elichak@gmail.com (Liz Chak)'


import time
import syck
import sys

import report
import paths
import tp
import ts
import os
from os.path import join, getsize
import stat
import string
import urlparse 


# Number of times to run startup test (Ts)
TS_NUM_RUNS = 5

# Number of times the page load test (Tp) loads each page in the test.
TP_NUM_CYCLES = 7

# Resolution of counter sample data for page load test (Tp), in seconds
# (For example, if TP_RESOLUTION=1, sample counters every 1 second
TP_RESOLUTION = 1

# Since python doesn't have 'switch' statements, this class
# aims to duplicate C's original switch functionality and
# structure with reasonable accuracy

class switch(object):
    def __init__(self, value):
        self.value = value
        self.fall = False

    def __iter__(self):
        """Return the match method once, then stop"""
        yield self.match
        raise StopIteration
    
    def match(self, *args):
        """Indicate whether or not to enter a case suite"""
        if self.fall or not args:
            return True
        elif self.value in args: 
            self.fall = True
            return True
        else:
            return False

# This handles any exception
       
def ExpHandler(*posargs):

    def nestedhandler(func,exptuple, *pargs, **kwargs):
        """ Function that creates a nested exception handler from
        the passed exception tuple """

        exp, handler = exptuple[0]
        try:
            if len(exptuple)==1:
                func(*pargs, **kwargs)
            else:
                nestedhandler(func,exptuple[1:], *pargs, **kwargs)
        except exp, e:
            if handler:
                handler(e)
            else:
                print e.__class__.__name__,':',e
                sys.stdout.flush()
        
    def wrapper(f):
        t = tuple(item for item in posargs[0]
            if issubclass(item,Exception)) or (Exception,)
        def newfunc(*pargs, **kwargs):
            try: 
                f(*pargs, **kwargs)
            except t, e:
                # Only inform the user that the exception occured
                print e.__class__.__name__,':',e
                sys.stdout.flush()

        return newfunc
    return wrapper


def myhandler(e):
    print 'Caught exception!', e

@ExpHandler((KeyError, Exception))
def test_file(filename):
  """Runs the Ts and Tp tests on the given config file and generates a report.
  
  Args:
    filename: the name of the file to run the tests on
  """
  
  test_configs = []
  test_names = []
  title = ''
  filename_prefix = ''
  
  # Read in the profile info from the YAML config file
  
  try:
        print "checking... reading %s" %filename
        if not os.path.exists(filename):
           error = "Error... " + filename + " doesn't exist, please create it in the same directory!" 
           sys.exit(error) 
  	config_file = open(filename, 'r')
  	yaml = syck.load(config_file)
  	config_file.close()
  	
  	print "checking... Values in %s" %filename
  	sys.stdout.flush()
  	for item in yaml:
  	      	
		c = item
		for case in switch(c):
			print "checking... Value of item in %s: %s" %(filename, item)
			if case('filename'):
				filename_prefix = yaml[item]
				print "checking... filename: %s" %yaml[item]
				break
  	  		if case('title'): # note the * for unpacking as arguments
  	  			title = yaml[item]
  		      		print "checking... title: %s" %yaml[item]
  		      		break
  	  		if case(): # normal argument passing style also applies
  		      		print "checking... Values of preferences, extensions, firefox"
  		      		new_config = [yaml[item]['preferences'],
					      yaml[item]['extensions'],
					      yaml[item]['firefox']]
				print "checking... preferences: %s" %yaml[item]['preferences']
				print "checking... extensions: %s" %yaml[item]['extensions']
				print "checking... firefox: %s" %yaml[item]['firefox']
				if not os.path.exists(yaml[item]['firefox']):
  					error = "Error... Please check that the firefox path in "+filename+" points to the desired instance: " + yaml[item]['firefox']
  					sys.exit(error)
			        test_configs.append(new_config)
  	                      	test_names.append(item)
  		      		break
  	sys.stdout.flush()
  	print "processing... Performance Testing is starting"
  except IOError:
  	IO_error = "Error... Please check your file:" + filename
  	sys.exit(IO_error)
  config_file.close()
  
  # Run startup time test
  ts_times = ts.RunStartupTests(paths.BASE_PROFILE_DIR,
                                test_configs,
                                TS_NUM_RUNS)
  
  # Run page load test.  For possible values of counters argument, see
  # http://technet2.microsoft.com/WindowsServer/en/Library/86b5d116-6fb3-427b-af8c-9077162125fe1033.mspx?mfr=true
  (tp_times, tp_counters) = tp.RunPltTests(paths.BASE_PROFILE_DIR,
                                           test_configs,
                                           TP_NUM_CYCLES,
                                           ['Private Bytes', 'Working Set', '% Processor Time'],
                                           TP_RESOLUTION)
  
  # Generate a report of the results.
  report.GenerateReport(title,
                        filename_prefix,
                        test_names,
                        ts_times,
                        tp_times,
                        tp_counters,
                        TP_RESOLUTION)



@ExpHandler((IOError, Exception))
def checkPaths():
  """
  This will check if the paths in paths.py exist
  
  """
  
  # Testing code for checking if directories exist
  # and making directories if they don't exist

  """
  created = False
  
  while not created:
	resp = raw_input("Which directory? ")
	dirname = resp
	if not os.path.isdir("./" + dirname + "/"):
		os.mkdir("./" + dirname + "/")
		print "Dir %s is created" % resp
		created = True
	else:	
		print "sorry, the dir %s existed already" % resp
		created = False
	sys.stdout.flush()
  """

  # Checks if the following directories exist (initialized in paths.py)

  path_dirs = [paths.BASE_PROFILE_DIR, paths.REPORTS_DIR] 
  files =[paths.INIT_URL, paths.TP_URL, paths.TS_URL]
  
  # first attempt to check the dirs of the paths
  # works fine
  """
  for dir in path_dirs:
	print "checking... Dir: %s" % dir 
	if not os.path.isdir(dir):
		print "Error... Dir doesn't Exist: %s" % dir 
		sys.stdout.flush()
		error = "Error... Please check that " + dir +"is created or exist or change path in paths.py to correct path"  
		sys.exit(error)
	else:	
		print "checking... Exist: Dir %s" % dir
  sys.stdout.flush()
  """
  
  # Checks if paths in paths.py exist

  for dir in path_dirs:
    #head, tail = os.path.split(dir)
    head = dir
    if not os.path.exists(head): 
	print "Error... Dir doesn't Exist: %s" % head
	error = "Error... Please check that" + head + "is created or exist or change path in paths.py to correct path" 
	sys.exit(error)
    elif os.path.isdir(head):
	print "checking... Exist: Dir %s" % head
  sys.stdout.flush()
	

  #check uf contents of base_profile_dir exist
  
  if os.path.isdir(paths.BASE_PROFILE_DIR):
	for root, dirs, files in os.walk(paths.BASE_PROFILE_DIR):
	    print root, "consumes",
	    print sum(getsize(join(root, name)) for name in files),
	    print "bytes in", len(files), "non-directory files"
            if 'CVS' in dirs:
                 dirs.remove('CVS')  # don't visit CVS directories 
        if sum(getsize(join(root, name)) for name in files) == 0:
            error = "Error... Please check that you have files in " + paths.BASE_PROFILE_DIR
            sys.exit(error)
  elif not os.path.isdir(paths.BASE_PROFILE_DIR):
       error = "Error... Please check that " + paths.BASE_PROFILE_DIR + " exist"
       sys.exit(error)
  sys.stdout.flush()
         
  #problem with parsing file urls (file:///c:/)
  """
  for file in files:
  	url = urlparse.urlparse(file)

	if not os.path.exists(file):
		error = "Aborting... Please check if the path in paths.py for " + file + " is right"
		sys.exit(error)
	else:
		print "checking... Exist: File %s" %file
  	
  sys.stdout.flush()
  """
if __name__=='__main__':  	
 
  #check if paths exist
  checkPaths();
  # Read in each config file and run the tests on it.
  for i in range(1, len(sys.argv)):
    test_file(sys.argv[i])
    

